/*  TilEm, TI-Linux Emulator
 *  Copyright (C) 2001 Solignac Julien <x1cygnus@xcalc.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#define maxtime 170
/* 70 is enough for the TI-83+, 83 is a bit slower though */

#include<string.h>
#include<stdlib.h>
#include<stdio.h>

#include "../Z80.h"
#include "inter.h"

void create_savedlg(void);


byte prgmname[8], *linktmp = NULL;
byte xsignal[20] = {0x83, 0x00, 0x00, 0x00};
byte timeout = 0, mode; 
unsigned short hstatus;

static byte count = 8;

/* used by handlelink */
static byte *sendrec;
static unsigned short decount, checksum;


/* Returns 1 one when finished, -1 on error */
char sendbyte(byte line, byte tosend)
{
	extern byte pclnk;

	static byte status;
	static byte send;

	if (count == 0x08) {
		printf("[Snd] 0x%02X\n", tosend);
		send = tosend;
		count = status = 0;
	}

	switch(status) {
		case 0:
			if (line != 0x03) {
				pclnk = 0x03;
				break;
			}
			pclnk = 1 + ((~send >> count) & 0x01);
			status++;
			break;

		case 1:
			if (!line) {
				pclnk = 0x03;
				status++;
			}
			break;

		case 2:
			if (line == 0x03) {
				count++;
				status = 0;
			}
	}

	return((count & 0x08)?1:0);
}


int recbyte(byte line)
{
	extern byte pclnk;

	static byte status = 0;
	static byte rec;

	if (count == 0x08)
		count = status = rec = 0;

	switch(status) {
		case 0:
			if (line == 0x03)
				break;
			rec = rec | ((line & 0x01) << count);
			pclnk = (~line) & 0x03;
			status++;
			break;

		case 1:
			if (!line)
				break;
			pclnk = 0x03;
			status++;

		case 2:
			if (line != 0x03)
				break;
			status = 0;
			count++;
	}

	return((count & 0x08)?rec:-1);
}


void setrecv(short num, byte *flow, byte chksum)
{
	timeout = maxtime;
	mode = 1;
	sendrec = flow;
	checksum = 0;
	decount = num + 2 * chksum;
	hstatus++;
}


void setsend(short num, byte *data, byte chksum)
{
	timeout = maxtime;
	mode = 2;
	sendrec = data;
	checksum = 0;
	decount = num + 2 * chksum;
	hstatus++;
}


void handlelink(byte line)
{
	extern struct hardware *hw;

	static unsigned short xchk;

	short i;

	if (!timeout)
		mode = 0;

	switch (mode) {
		case 0:
			count = 0x08;
			if (line != 0x03) {
				setrecv(4, xsignal, 0);
				hstatus = 0;

				if (hw->model == '3') {
					setrecv(4 + 11, xsignal, 1);
					hstatus = 2;
				}
			}
			break;

		case 1:
			timeout--;
			i = recbyte(line);
			if (i != -1) {
				printf("[Rec] 0x%02X\n", i);
				timeout = maxtime;
				*sendrec++ = i;
				if (decount > 2)
					checksum += i;
				else
					xchk = (xchk >> 8) + (i << 8);

				decount--;
			}
			break;

		case 2:
			timeout--;
			if (sendbyte(line, *sendrec)) {
				timeout = maxtime;
				decount--;
				if (decount)
					sendbyte(0x03, *(++sendrec));
			}
	}	

	if (timeout && !decount) {
		mode = 3;
		timeout = 0;
		switch (hstatus) {
		/* Recieve File */
			case 0:
				if (xsignal[1] == 0x68) {
					xsignal[1] = 0x56;
					setsend(4, xsignal, 0);
				}
				break;

			case 1:
				setrecv(4 + 11, xsignal, 1);
				break;

			case 2:
				if ((xsignal[4 + 2]  - 0x05) <= 0x01) {
					strncpy(prgmname, xsignal + 7, 8);	

					xsignal[2] = xsignal[3] = xsignal[6] = xsignal[7] = 0x00;
					xsignal[4] = xsignal[0];
					xsignal[1] = 0x56;
					xsignal[5] = 0x09;

					setsend(8, xsignal, 0);
				}
				break;

			case 3:
				setrecv(8, xsignal, 0);
				break;

			case 4:
				i = xsignal[6] + (xsignal[7] << 8);
				if (linktmp)
					free(linktmp);
				if ((linktmp = malloc(i + 2)))
					setrecv(i, linktmp, 1);
				break;

			case 5:
				if (checksum != xchk)
					break;

				setsend(4, xsignal, 0);
				break;	

			case 6:
				setrecv(4, xsignal, 0);
				break;

			case 7:
				xsignal[1] = 0x56;
				setsend(4, xsignal, 0);

				create_savedlg();
				break;

		/* Send File */
			case 9:
				setsend(4 + 0x0B + 2, xsignal, 0);
				break;

			case 10:
				setrecv(8, xsignal, 0);
				break;

			case 11:
				setsend(8 + *(linktmp + 6) + (*(linktmp + 7) << 0x08) + 2, linktmp, 0);
				break;	

			case 12:
				setrecv(4, xsignal, 0);
				break;

			case 13:
				xsignal[1] = 0x92;
				setsend(4, xsignal, 0);
		}
	}
}
