/*  Tilem, TI-Linux Emulator
 *  Copyright (C) 2001 Solignac Julien <x1cygnus@xcalc.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include<stdlib.h>

extern struct debuginfo *debugdat;
void refreshdebugger(unsigned short xpc);

static struct chbxmem hexdat;


unsigned short getreg(Z80_Regs * R, byte i)
{
	switch (i) {
		case 0:
			return(R->AF.D);
		case 1:
			return(R->HL.D);
		case 2:
			return(R->DE.D);
		case 3:
			return(R->BC.D);
		case 4:
			return(R->PC.D);
		case 5:
			return(R->IY.D);
		case 6:
			return(R->AF2.D);
		case 7:
			return(R->HL2.D);
		case 8:
			return(R->DE2.D);
		case 9:
			return(R->BC2.D);
		case 10:
			return(R->SP.D);
		case 11:
			return(R->IX.D);
	}

	return(0x0000);
}

void setreg(Z80_Regs * R, byte i, unsigned short v)
{
	switch (i) {
		case 0:
			R->AF.D = v;
			break;
		case 1:
			R->HL.D = v;
			break;
		case 2:
			R->DE.D = v;
			break;
		case 3:
			R->BC.D = v;
			break;
		case 4:
			R->PC.D = v;
			break;
		case 5:
			R->IY.D = v;
			break;
		case 6:
			R->AF2.D = v;
			break;
		case 7:
			R->HL2.D = v;
			break;
		case 8:
			R->DE2.D = v;
			break;
		case 9:
			R->BC2.D = v;
			break;
		case 10:
			R->SP.D = v;
			break;
		case 11:
			R->IX.D = v;
			break;
	}
}


static void interfunc(GtkWidget *dialog)
{
	unsigned short x;
	char *endp, *begp;

	begp = gtk_entry_get_text(GTK_ENTRY(hexdat.ent));
	x = (unsigned short) strtol(begp, &endp, 16);

	if ((*endp == '\0') && (*begp != '\0'))
		(hexdat.func)(x, hexdat.i);

	gtk_widget_destroy(GTK_WIDGET(dialog));
}


void create_hexbox(char *title, char *label, char *value, char maxlen, ptrfunc func, unsigned short data)
{
	GtkWidget *dlg, *dlgstf1, *frame, *lblbx, *entry, *dialog_aa, *bbox, *button1, *button2;

	dlg = gtk_dialog_new();
		gtk_window_set_title(GTK_WINDOW(dlg), title);
		gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);

	button1 = gtk_button_new_with_label("OK");
	gtk_widget_show(button1);

	button2 = gtk_button_new_with_label("Cancel");
	gtk_widget_show(button2);

	bbox = gtk_hbutton_box_new();
		gtk_box_pack_start(GTK_BOX(bbox), button1, FALSE, FALSE, 0);
		gtk_box_pack_end(GTK_BOX(bbox), button2, FALSE, FALSE, 0);
	gtk_widget_show(bbox);

	dialog_aa = GTK_DIALOG(dlg)->action_area;
		gtk_container_set_border_width(GTK_CONTAINER (dialog_aa), 10);
		gtk_box_pack_start(GTK_BOX(dialog_aa), bbox, FALSE, TRUE, 0);
	gtk_widget_show(dialog_aa);

	entry = gtk_entry_new_with_max_length(maxlen);
		gtk_entry_set_text(GTK_ENTRY(entry), value);
		gtk_entry_select_region(GTK_ENTRY(entry), 0, maxlen);
	gtk_widget_show(entry);

	lblbx = gtk_hbox_new(FALSE, 0);
		gtk_container_set_border_width(GTK_CONTAINER(lblbx), 5);
		gtk_box_pack_start(GTK_BOX(lblbx), entry, TRUE, TRUE, 0);
	gtk_widget_show(lblbx);

	frame = gtk_frame_new(label);
		gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
		gtk_container_add(GTK_CONTAINER(frame), lblbx);
	gtk_widget_show(frame);

	dlgstf1 = GTK_DIALOG(dlg)->vbox;
		gtk_box_pack_start(GTK_BOX(dlgstf1), frame, TRUE, TRUE, 0);
	gtk_widget_show(dlgstf1);

		gtk_signal_connect_object(GTK_OBJECT(entry), "activate", GTK_SIGNAL_FUNC(interfunc), GTK_OBJECT(dlg));
		gtk_signal_connect_object(GTK_OBJECT(button1), "clicked", GTK_SIGNAL_FUNC(interfunc), GTK_OBJECT(dlg));
		gtk_signal_connect_object(GTK_OBJECT(button2), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(dlg));
		gtk_signal_connect_object(GTK_OBJECT(dlg), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(dlg));
	gtk_widget_show(dlg);

	hexdat.i = data;
	hexdat.ent = entry;
	hexdat.func = func;

	gtk_widget_grab_focus(entry);
}
