/*  Tilem, TI-Linux Emulator
 *  Copyright (C) 2001 Solignac Julien <x1cygnus@xcalc.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

G_LOCK_EXTERN(halted);

struct debuginfo *debugdat;
static char *rlabels[12] = {"AF", "HL", "DE", "BC", "PC", "IY", "AF'", "HL'", "DE'", "BC'", "SP", "IX"};

unsigned short hexloc = 0x0000;
byte writestk = 0;

/************ Protos *************/
int Z80_Dasm(unsigned char *buffer, char *dest, unsigned PC);
void chreg(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
void create_pixmap(char *filename, GtkWidget *parent, GdkPixmap **pixmap, GdkBitmap **mask);

unsigned short getreg(Z80_Regs * R, byte i);
void setreg(Z80_Regs * R, byte i, unsigned short v);

GtkWidget* create_regchbox(byte rgpara);
void create_hexbox(char *title, char *label, char *value, char maxlen, ptrfunc func, unsigned short data);
void refreshdebugger(unsigned short xpc);
/********* End Protos ************/


void debugend (GtkObject *debugw)
{
	extern short Z80_Trace;
	extern byte enabled;

	Z80_Trace = enabled = 0;
	gtk_widget_destroy(GTK_WIDGET(debugw));

/* find a way to give back focus */
	free(debugdat);
	G_UNLOCK(halted);
}

void stepovr(GtkWidget *widget, GtkObject *debugw)
{
	extern struct hardware *hw;
	extern opcode_fn opcode_main[256];
	extern unsigned short stepbpx;

	byte i, b[5];
	char st[20];
	unsigned short xpc;
	unsigned opcode;

	xpc = debugdat->RX->PC.D;

	for (i = 0; i < 5; i++)
		b[i] = Z80_RDMEM(xpc + i);

	stepbpx = xpc + Z80_Dasm(b, st, xpc);

	while ((debugdat->RX->PC.D) == xpc) {
		opcode = Z80_RDMEM(debugdat->RX->PC.D);
		debugdat->RX->PC.D++;
		(*(opcode_main[opcode]))();
	}

	if (debugdat->RX->PC.D == stepbpx) {
		stepbpx = 0;
		refreshdebugger(debugdat->RX->PC.D);

	} else {
		gtk_widget_destroy(GTK_WIDGET(debugw));
	}
}


void dostep(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	G_UNLOCK(halted);
}


void reficons(byte sel)
{
	extern unsigned short bpx[maxbpx];
	extern byte numbpx;

	byte i, j;
	GdkPixmap *pix1, *pix2, *pix3, *pixc;
	GdkBitmap *msk1, *msk2, *msk3, *mskc;

	create_pixmap("pc.xpm", GTK_WIDGET(debugdat->dasm), &pix1, &msk1);
	create_pixmap("bpx.xpm", GTK_WIDGET(debugdat->dasm), &pix2, &msk2);
	create_pixmap("pcbpx.xpm", GTK_WIDGET(debugdat->dasm), &pix3, &msk3);
	create_pixmap("clear.xpm", GTK_WIDGET(debugdat->dasm), &pixc, &mskc);

	for (j = 0; j < listlen; j++) {
		gtk_clist_set_pixmap(GTK_CLIST(debugdat->dasm), j, 0, pixc, mskc);
		if (debugdat->lstpc[j] == debugdat->RX->PC.D) {
			gtk_clist_set_pixmap(GTK_CLIST(debugdat->dasm), j, 0, pix1, msk1);
			if (sel)
				gtk_clist_select_row(GTK_CLIST(debugdat->dasm), j, 0);
		}

		for (i = 0; i < numbpx; i++) {
			if (bpx[i] == debugdat->lstpc[j]) {
				if (debugdat->lstpc[j] == debugdat->RX->PC.D)
					gtk_clist_set_pixmap(GTK_CLIST(debugdat->dasm), j, 0, pix3, msk3);
				else
					gtk_clist_set_pixmap(GTK_CLIST(debugdat->dasm), j, 0, pix2, msk2);
			}
		}
	}

	gdk_pixmap_unref(pix1); gdk_pixmap_unref(pix2); gdk_pixmap_unref(pix3); gdk_pixmap_unref(pixc);
	gdk_bitmap_unref(msk1); gdk_bitmap_unref(msk2); gdk_bitmap_unref(msk3); gdk_bitmap_unref(mskc);
}


void setbpx(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	extern unsigned short bpx[maxbpx];
	extern byte numbpx;

	gchar *text, *endp;
	unsigned short x;
	byte i, z;

	gtk_clist_get_text(GTK_CLIST(debugdat->dasm), debugdat->xselected, 1, &text);
	x = (unsigned short) strtol(text, &endp, 16);

	for (i = 0; i < numbpx; i++) {
		if (bpx[i] == x) {
			for (z = i; z < maxbpx - 1; z++)
				bpx[z] = bpx[z + 1];
			numbpx--;
			reficons(0);
			return;
		}
	}

	if (numbpx == maxbpx) {
		fprintf(stderr, "** max breakpoints exceeded **\n");
		return;
	}

	bpx[numbpx] = x;
	numbpx++;
	reficons(0);
}


void selectline(GtkWidget *widget, gint row, gint col, GdkEventButton *event, gpointer data)
{
/* CLIST Doesn't provide a getrow function! */
	debugdat->xselected = row;
}


void refreshstk(GtkWidget *stkcode)
{
	extern struct hardware *hw;
	char str[2][6], *co[2];
	unsigned short i;

	gtk_clist_freeze(GTK_CLIST(stkcode));
	gtk_clist_clear(GTK_CLIST(stkcode));

	co[0] = str[0];
	co[1] = str[1];
	i = getreg(debugdat->RX, 10);
	while (i > 0x0010) {
		sprintf(str[0], "%04X:", i);
		sprintf(str[1], "%04X", Z80_RDMEM(i) + (Z80_RDMEM(i + 1) << 0x08));

		gtk_clist_append(GTK_CLIST(stkcode), co);
		i += 0x0002;
	}

	gtk_clist_thaw(GTK_CLIST(stkcode));
}


void refreshhex(GtkWidget *hexcode)
{
	extern struct hardware *hw;
	char str[10][9], *co[10];
	byte i, j, k;

	gtk_clist_freeze(GTK_CLIST(hexcode));
	gtk_clist_clear(GTK_CLIST(hexcode));

	for (i = 0; i < 10; i++)
		co[i] = str[i];
	str[9][8] = '\0';

	for (j = 0; j < 5; j++) {
		sprintf(str[0], "%04X", hexloc + 0x08 * j);
		for (i = 1; i < 9; i++) {
			k = Z80_RDMEM(hexloc + i - 1 + 0x08 * j);
			sprintf(str[i], "%02X", k);

			if ((k >= 0x20) && (k < 0x7E))
				str[9][i - 1] = k;
			else
				str[9][i - 1] = '.';

		}
	
		gtk_clist_append(GTK_CLIST(hexcode), co);
	}

	gtk_clist_thaw(GTK_CLIST(hexcode));

}


void clhex(unsigned short x, unsigned short i)
{
	hexloc = x;
	refreshhex(debugdat->hexcode);
}


void cchex(unsigned short x, unsigned short i)
{
	extern struct hardware *hw;

	Z80_WRMEM(i, (byte) x);
	refreshhex(debugdat->hexcode);
}


void hexchange(GtkWidget *widget, gint row, gint col, GdkEventButton *event, gpointer data)
{
	extern struct hardware *hw;

	unsigned short z;
	char label[15], datum[5];

	gtk_clist_unselect_row(GTK_CLIST(widget), row, col);

	z = hexloc + row * 0x08 + col - 1;
	if ((col > 0) && (col < 9) && (z >= 0x8000)) {
		sprintf(label, "Byte at 0x%04X", z);
		sprintf(datum, "%02X", Z80_RDMEM(z));

		create_hexbox("Change Memory", label, datum, 2, (ptrfunc) cchex, z);
	} else if (!col) {
		sprintf(datum, "%04X", hexloc);
		create_hexbox("Code Panel", "Goto", datum, 4, (ptrfunc) clhex, 0);
	}
}



void ccasm(unsigned short x, unsigned short i)
{
	refreshdebugger(x);
}

void chkdbgwin (GtkWidget *debugw, GdkEventKey *event, gpointer user_data)
{
	char data[5];
	unsigned short svpc;

	if (event->keyval == GDK_F5) {
		gtk_widget_destroy(GTK_WIDGET(debugw));

	} else if (event->keyval == GDK_F7) {
		G_UNLOCK(halted);

	} else if (event->keyval == GDK_F2) {
		setbpx(NULL, NULL, NULL);

	} else if (event->keyval == GDK_Down) {
		if (debugdat->xselected == listlen - 1) {
			svpc = debugdat->lstpc[listlen - 1];
			debugdat->lstpc[listlen - 1] = 0x0000;
			refreshdebugger(svpc);
			debugdat->xselected = 0;
		}
		gtk_clist_select_row(GTK_CLIST(debugdat->dasm), debugdat->xselected + 1, 0);

	} else if (event->keyval == GDK_Up) {
		if (!debugdat->xselected) {
			refreshdebugger(debugdat->lstpc[0] - 1);
			debugdat->xselected = 1;
		}
		gtk_clist_select_row(GTK_CLIST(debugdat->dasm), debugdat->xselected - 1, 0);

	} else if (event->keyval == GDK_g) {
		snprintf(data, 5, "%04X", debugdat->RX->PC.D);
		create_hexbox("Disassembly", "Goto", data, 4, (ptrfunc) ccasm, 0);
	}
		
}

void ccreg(unsigned short x, unsigned short i)
{
	setreg(debugdat->RX, i, x);
	refreshdebugger(debugdat->RX->PC.D);
}

void chreg(GtkWidget *widget, GdkEventKey *event, gpointer rselected)
{
	char label[15], data[5];

	snprintf(label, 15, "Value for %s:", rlabels[(int) rselected]);
	snprintf(data, 5, "%04X", getreg(debugdat->RX, (int) rselected));

	create_hexbox("Change Register", label, data, 4, (ptrfunc) ccreg, (int) rselected);
	return;
}


void create_debugger(Z80_Regs *R)
{
	extern unsigned short stepbpx;

	GtkWidget *debugwin;
	GtkWidget *trio, *duo, *regbox, *toolbar, *xframe, *hexcode, *regev, *dasmlist, *stkframe, *bduo, *stkcode, *stkscroll;
	byte i;
	
	GdkPixmap *pixmap;
	GdkBitmap *mask;
	GtkWidget *icon;

	stepbpx = 0;

	debugdat = malloc(sizeof(struct debuginfo));
	for (i = 0; i < listlen; i++)
		debugdat->lstpc[i] = 0xFFFF;

	debugwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_object_set_data(GTK_OBJECT(debugwin), "debugwin", debugwin);
		gtk_window_set_modal(GTK_WINDOW(debugwin), TRUE);
		gtk_widget_set_usize(debugwin, 420, 280);
/*		gtk_window_set_policy(GTK_WINDOW(debugwin), FALSE, FALSE, TRUE);
 *	unfortunatly, also changes window's border... */
		gtk_window_set_title(GTK_WINDOW(debugwin), "Tilem Debugger");

	dasmlist = gtk_clist_new(3);
		gtk_widget_set_name(dasmlist, "monow");

		gtk_clist_set_column_width(GTK_CLIST(dasmlist), 0, 15);
		gtk_clist_set_column_width(GTK_CLIST(dasmlist), 1, 50);
		gtk_clist_set_column_width(GTK_CLIST(dasmlist), 2, 120);
		gtk_clist_column_titles_hide(GTK_CLIST(dasmlist));

//		gtk_widget_set_sensitive(dasmlist, FALSE);
		GTK_WIDGET_UNSET_FLAGS(dasmlist, GTK_CAN_FOCUS);
	gtk_widget_show(dasmlist);

	regbox = gtk_table_new(6, 2, TRUE);
		for (i = 0; i < 12; i++) {
			regev = gtk_event_box_new();
				debugdat->regl[i] = gtk_label_new("---");
					gtk_container_add(GTK_CONTAINER(regev), debugdat->regl[i]);
				gtk_widget_show(debugdat->regl[i]);

				if (i != 11) {
					gtk_widget_set_events(regev, GDK_BUTTON_PRESS_MASK);
					gtk_signal_connect(GTK_OBJECT(regev), "button_press_event", GTK_SIGNAL_FUNC(chreg), (gpointer) (int) i);
				}
			gtk_widget_show(regev);

			gtk_table_attach(GTK_TABLE(regbox), regev, i / 6, i / 6 + 1, i % 6, i % 6 + 1, 0, 0, 3, 0);
		}
	gtk_widget_show(regbox);

	xframe = gtk_frame_new("Registers");
		gtk_container_set_border_width(GTK_CONTAINER(xframe), 5);
		gtk_container_add(GTK_CONTAINER(xframe), regbox);
	gtk_widget_show(xframe);

//	hexduo = gtk_vscrollbar_new(GTK_ADJUSTMENT(adjust));
//	gtk_widget_show(hexduo);

	duo = gtk_hbox_new(FALSE, 0);
		gtk_box_pack_start(GTK_BOX(duo), dasmlist, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(duo), xframe, FALSE, FALSE, 0);
//		gtk_box_pack_end(GTK_BOX(duo), hexduo, FALSE, FALSE, 0);
	gtk_widget_show(duo);


	toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
		gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NONE);

		create_pixmap("step.xpm", GTK_WIDGET(toolbar), &pixmap, &mask);
		icon = gtk_pixmap_new(pixmap, mask);
		gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Step (F7)", "Private", icon, GTK_SIGNAL_FUNC(dostep), NULL);

		create_pixmap("stepovr.xpm", GTK_WIDGET(toolbar), &pixmap, &mask);
		icon = gtk_pixmap_new(pixmap, mask);
		gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Step Over", "Private", icon, GTK_SIGNAL_FUNC(stepovr), GTK_OBJECT(debugwin));

		create_pixmap("setbpx.xpm", GTK_WIDGET(toolbar), &pixmap, &mask);
		icon = gtk_pixmap_new(pixmap, mask);
		gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Toggle Breakpoint (F2)", "Private", icon, GTK_SIGNAL_FUNC(setbpx), dasmlist);
	gtk_widget_show(toolbar);

	hexcode = gtk_clist_new(10);
		gtk_widget_set_name(hexcode, "monow");

		gtk_clist_set_column_width(GTK_CLIST(hexcode), 0, 40);
		for (i = 1; i < 10; i++)
			gtk_clist_set_column_width(GTK_CLIST(hexcode), i, 16);

		gtk_clist_column_titles_hide(GTK_CLIST(hexcode));
		GTK_WIDGET_UNSET_FLAGS(hexcode, GTK_CAN_FOCUS);
	gtk_widget_show(hexcode);

	xframe = gtk_frame_new("Memory");
		gtk_container_set_border_width(GTK_CONTAINER(xframe), 5);
		gtk_container_add(GTK_CONTAINER(xframe), hexcode);
	gtk_widget_show(xframe);



	stkcode = gtk_clist_new(2);
		gtk_clist_column_titles_hide(GTK_CLIST(stkcode));
		GTK_WIDGET_UNSET_FLAGS(stkcode, GTK_CAN_FOCUS);
		gtk_clist_set_column_width(GTK_CLIST(stkcode), 0, 30);
		gtk_clist_set_column_width(GTK_CLIST(stkcode), 1, 25);

	gtk_widget_show(stkcode);

	stkscroll = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(stkscroll), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
		gtk_container_add(GTK_CONTAINER(stkscroll), stkcode);
	gtk_widget_show(stkscroll);




	stkframe = gtk_frame_new("Stack");
		gtk_container_set_border_width(GTK_CONTAINER(stkframe), 5);
		gtk_container_add(GTK_CONTAINER(stkframe), stkscroll);
	gtk_widget_show(stkframe);

	bduo = gtk_hbox_new(FALSE, 0);
		gtk_box_pack_start(GTK_BOX(bduo), xframe, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(bduo), stkframe, FALSE, FALSE, 0);
	gtk_widget_show(bduo);

	trio = gtk_vbox_new(FALSE, 0);
		gtk_box_pack_start(GTK_BOX(trio), toolbar, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(trio), duo, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(trio), bduo, TRUE, TRUE, 0);
	gtk_widget_show(trio);

		gtk_container_add(GTK_CONTAINER(debugwin), trio);

		gtk_signal_connect(GTK_OBJECT(debugwin), "key_press_event", GTK_SIGNAL_FUNC(chkdbgwin), NULL);
		gtk_signal_connect(GTK_OBJECT(hexcode), "select_row", GTK_SIGNAL_FUNC(hexchange), NULL);
		gtk_signal_connect(GTK_OBJECT(dasmlist), "select_row", GTK_SIGNAL_FUNC(selectline), NULL);
		gtk_signal_connect_object(GTK_OBJECT(debugwin), "destroy", GTK_SIGNAL_FUNC(debugend), GTK_OBJECT(debugwin));
	gtk_widget_show(debugwin);

	refreshhex(hexcode);
	debugdat->hexcode = hexcode;
	debugdat->stkcode = stkcode;
	debugdat->dasm = dasmlist;
	debugdat->RX = R;

	refreshstk(stkcode);
}



void refreshdebugger(unsigned short xpc)
{
	extern struct hardware *hw;

	short spc;
	byte i, j, b[5];
	char *str[3], st[20], addr[5];

	static unsigned short oldstack = 0xFFFF;

	if ((writestk) || (oldstack != getreg(debugdat->RX, 10)))
		refreshstk(debugdat->stkcode);
	oldstack = getreg(debugdat->RX, 10);
	writestk = 0;
/* INC SP instructions... */


	for (j = 0; j < listlen; j++) {
		if (debugdat->lstpc[j] == xpc)
			xpc = debugdat->lstpc[0];
	}

/* Has the advantage of preventing the list from flickering,
 * but then we miss out all dynamically changed code... */
	if (debugdat->lstpc[0] != xpc) {
		gtk_clist_freeze(GTK_CLIST(debugdat->dasm));
		gtk_clist_clear(GTK_CLIST(debugdat->dasm));
		for (j = 0; j < listlen; j++) {
			debugdat->lstpc[j] = xpc;

			for (i = 0; i < 5; i++)
				b[i] = Z80_RDMEM(xpc + i);

			snprintf(addr, 5, "%04X", xpc);	

			str[0] = NULL;
			str[1] = addr;
			str[2] = st;
			spc = Z80_Dasm(b, st, xpc);

			gtk_clist_append(GTK_CLIST(debugdat->dasm), str);
			xpc += spc;

/*			if (gtk_clist_row_is_visible(GTK_CLIST(dasmlist), j) != GTK_VISIBILITY_FULL)
				break;*/
		}
		gtk_clist_thaw(GTK_CLIST(debugdat->dasm));
	}

	reficons(1);

	for (i = 0; i < 12; i++) {
		snprintf(st, 10, "%s: %04X", rlabels[i], getreg(debugdat->RX, i));
		gtk_label_set_text(GTK_LABEL(debugdat->regl[i]), st);
	}

/* Has No Effect Whatsoever
 * gtk_label_set_justify(debugdat->regl[i], GTK_JUSTIFY_LEFT); */
}
