/*  Tilem, TI-Linux Emulator
 *  Copyright (C) 2001 Solignac Julien <x1cygnus@xcalc.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

void loadapp(char *file);

byte tiheader[11] = {'*', '*', 'T', 'I', '8', '3', '*', '*', 0x1A, 0x0A, 0x00};
byte ticomment[42] = "File created with Tilem\0                 ";

static char *lastfile = NULL;


void setfilename(GtkWidget *filew)
{
	if (lastfile)
		gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew), lastfile);
}

char *getfilename(GtkFileSelection *widget)
{
	if (lastfile)
		free(lastfile);
	lastfile = strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(widget)));
	return(lastfile);
}


void savedestroy(GtkWidget *widget, GtkFileSelection *filewin)
{
	G_LOCK_EXTERN(running);
	extern byte *linktmp;

	if (linktmp)
		free(linktmp);
	linktmp = NULL;

	G_UNLOCK(running);
	gtk_widget_destroy(GTK_WIDGET(filewin));
}

void savefile(GtkWidget *w, GtkFileSelection *filewin)
{
	extern byte *linktmp, prgmname[8];

	FILE *dump;
	unsigned short filesize, checksum, x;

	if ((dump = fopen(getfilename(filewin), "w"))) {
		puts("Correct File Name");

		filesize = *linktmp + (*(linktmp + 1) << 0x08);

		fwrite(tiheader, 1, 11, dump);
		fwrite(ticomment, 1, 42, dump);

/* Avoid Endian Problems */
		putc((filesize + 2 + 15) & 0xFF, dump);
		putc((filesize + 2 + 15) >> 0x08, dump);

		putc(0x0B, dump);
		putc(0x00, dump);

		checksum = 0x0B + 0x06;
		checksum += 2 * (((filesize + 2) & 0xFF) + ((filesize + 2) >> 0x08));
		for (x = 0; x < 8; x++)
			checksum += prgmname[x];
		for (x = 0; x < filesize + 2; x++)
			checksum += *(linktmp + x);

/*		putc(0x00, dump);	
		putc(0x00, dump);	Not 8x */

		putc((filesize + 2) & 0xFF, dump);
		putc((filesize + 2) >> 0x08, dump);
		putc(0x06, dump);
		fwrite(prgmname, 1, 8, dump);

		putc((filesize + 2) & 0xFF, dump);
		putc((filesize + 2) >> 0x08, dump);

		fwrite(linktmp, 1, 2 + filesize, dump);

		putc(checksum & 0xFF, dump);
		putc(checksum >> 0x08, dump);

		fclose(dump);
	}
	savedestroy(w, filewin);
}

void inline create_savedlg(void)
{
	GtkWidget *filew;
	G_LOCK_EXTERN(running);


	gdk_threads_enter();
		filew = gtk_file_selection_new("Save file as");
			gtk_window_set_modal(GTK_WINDOW(filew), TRUE);
			gtk_signal_connect(GTK_OBJECT(filew), "destroy", (GtkSignalFunc) savedestroy, filew);
			gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->cancel_button), "clicked", (GtkSignalFunc) savedestroy, GTK_OBJECT(filew));

			gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->ok_button), "clicked", (GtkSignalFunc) savefile, filew);
			setfilename(filew);
		gtk_widget_show(filew);
	gdk_threads_leave();

	G_LOCK(running);
	G_LOCK(running);
}

void loadfile(GtkWidget *w, GtkFileSelection *filewin)
{
	extern byte *linktmp, xsignal[20];
	extern unsigned short hstatus;
	extern byte timeout, mode;

	FILE *load;
	byte is8x = 0;
	char checkhdr[8], *file;
	unsigned short filesize, x, chksum, chk2;


	file = getfilename(filewin);
	if (strstr(file, "app")) {
		loadapp(file);
		gtk_widget_destroy(GTK_WIDGET(filewin));
		return;
	}

	if ((load = fopen(file, "rb"))) {
		fread(checkhdr, 1, 8, load);
		if ((!strncmp(checkhdr, "**TI83", 6))) {
			if ((!strncmp(checkhdr, "**TI83F*", 8)))
				is8x = 2;

			fseek(load, 0x46 + is8x, SEEK_SET); /* 8x */
			
			filesize = getc(load);
			filesize += getc(load) << 0x08;

			if (linktmp)
				free(linktmp);
			if ((linktmp = malloc(8 + 2 + filesize + 2))) {
				fseek(load, 0x37, SEEK_SET);

			/* Get Variable Header */
				xsignal[0] = 0x03;
				xsignal[1] = 0xC9;
				fread(xsignal + 2, 1, 2 + 0x0B + 2, load);
				xsignal[2] = 0x0B; /* 8x */

				chksum = 0;
				for (x = 4; x < 4 + 0x0B; x++)
					chksum += xsignal[x];

				xsignal[4 + 0x0B + 0] = chksum & 0xFF;
				xsignal[4 + 0x0B + 1] = chksum >> 0x08;


				fseek(load, 0x44 + is8x, SEEK_SET); /* 8x */
				fread(linktmp + 6, 1, 4 + filesize, load);

				chk2 = 0;
				for (x = 8; x < 8 + filesize + 2; x++)
					chk2 += *(linktmp + x);

				chksum += chk2;
				chksum += *(linktmp + 6);
				chksum += *(linktmp + 7);
				chksum -= getc(load);
				chksum -= getc(load) << 0x08;
				chksum += 0x0B + is8x;	/* 8x */
	
				if (chksum & 0xFFFF) {
					free(linktmp);
					linktmp = NULL;

				} else {
					*(8 + 2 + linktmp + filesize + 0) =  chk2 & 0xFF;
					*(8 + 2 + linktmp + filesize + 1) =  chk2 >> 0x08;

					*(linktmp + 0) = 0x03; *(linktmp + 1) = 0x56;
					*(linktmp + 2) = 0x00; *(linktmp + 3) = 0x00;
					*(linktmp + 4) = 0x03; *(linktmp + 5) = 0x15;

					mode = 3;
					timeout = 70;
					hstatus = 9;
				}
			}
		}
		fclose(load);
	}

	gtk_widget_destroy(GTK_WIDGET(filewin));
}


void inline create_loaddlg(void)
{
	GtkWidget *filew;

	filew = gtk_file_selection_new("Load file or App");
		gtk_window_set_modal(GTK_WINDOW(filew), TRUE);
		gtk_signal_connect_object(GTK_OBJECT(filew), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(filew));
		gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filew)->cancel_button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(filew));

		gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->ok_button), "clicked", GTK_SIGNAL_FUNC(loadfile), GTK_OBJECT(filew));
		setfilename(filew);
	gtk_widget_show(filew);
}
