/*  Tilem, TI-Linux Emulator
 *  Copyright (C) 2001 Solignac Julien <x1cygnus@xcalc.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#define SHARE_DIR "/usr/share"

#include<unistd.h>
#include<stdio.h>
#include<string.h>
#include<pwd.h>
static char fpath[100];

char* ahome(char *text)
{
	uid_t uid;
	struct passwd *p;
 
	uid = getuid();
 	if (!(p = getpwuid(uid))) {
		fprintf(stderr, "** unable to obtain user info **");
		exit(-1);
	}

	snprintf(fpath, 100, "%s/.TilEm/%s", p->pw_dir, text);
	return(fpath);
}


char* ashare(char *file)
{
	snprintf(fpath, 100, "../pixmaps/%s", file);
	if (access(fpath, R_OK))
		snprintf(fpath, 100, "%s/tilem/pixmaps/%s", SHARE_DIR, file);

	if (access(fpath, R_OK) != 0) {
		fprintf(stderr, "*** unable to access '%s' ***\n", file);
		exit(-1);
	}

	return(fpath);
}


void create_pixmap (char *filename, GtkWidget *parent, GdkPixmap **pixmap, GdkBitmap **mask)
{
	*pixmap = gdk_pixmap_colormap_create_from_xpm(NULL, gtk_widget_get_colormap(GTK_WIDGET(parent)), mask, NULL, ashare(filename));
	if (!(*pixmap)) {
		fprintf(stderr, "*** unable to pixmap '%s' ***\n", filename);
		exit(-1);
	}
}


GtkWidget* make_pixmap (char *filename, GtkWidget *parent)
{
	GtkWidget *pixmap;
	GdkPixmap *pix;
	GdkBitmap *msk;

	create_pixmap(filename, GTK_WIDGET(parent), &pix, &msk);
	
	pixmap = gtk_pixmap_new (pix, msk);
	gdk_pixmap_unref(pix);
	gdk_bitmap_unref(msk);

	return(pixmap);
}
