/*  Tilem, TI-Linux Emulator
 *  Copyright (C) 2001 Solignac Julien <x1cygnus@xcalc.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>

#define		rWidth		0x60
#define		rHeight		0x40

#define		iWidth		rWidth * 2
#define		iHeight		rHeight * 2

static gboolean gpix_refresh (GtkWidget *widget);

// *fixme* refresh lcd on a executed instructions basis
GtkWidget *Screen;
guchar *greylcd = NULL;
guint timerid;
byte lcdshow, lcdactive = 1; 

extern byte *lcdbuf;

G_LOCK_DEFINE(scrref);



void create_keyb(GtkWidget *kscreen, GtkWidget *drawarea)
{
	extern struct hardware *hw;

	char filename[15];
	GtkWidget *vtrio, *htrio;
	GtkWidget *calcleft, *calcright, *calctop, *calcbot;

	snprintf(filename, 15, "x%c_left.xpm", hw->model);
	calcleft = make_pixmap(filename, kscreen);
	gtk_widget_show(calcleft);

	snprintf(filename, 15, "x%c_right.xpm", hw->model);
	calcright = make_pixmap(filename, kscreen);
	gtk_widget_show(calcright);

	htrio = gtk_hbox_new(FALSE, 0);
		gtk_box_pack_start(GTK_BOX(htrio), calcleft, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(htrio), drawarea, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(htrio), calcright, FALSE, FALSE, 0);
	gtk_widget_show(htrio);


	snprintf(filename, 15, "x%c_top.xpm", hw->model);
	calctop = make_pixmap(filename, kscreen);
	gtk_widget_show(calctop);

	snprintf(filename, 15, "x%c_bot.xpm", hw->model);
	calcbot = make_pixmap(filename, kscreen);
	gtk_widget_show(calcbot);

	vtrio = gtk_vbox_new(FALSE, 0);
		gtk_box_pack_start(GTK_BOX(vtrio), calctop, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(vtrio), htrio, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(vtrio), calcbot, FALSE, FALSE, 0);
	gtk_widget_show(vtrio);

	gtk_container_add(GTK_CONTAINER(kscreen), vtrio);
}


void keyin(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	extern struct hardware *hw;

/* Use this to get gtk's keycodes */
//	printf("[0x%04X]\n", event->keyval);

	switch (event->keyval) {
		case GDK_F11:
			invokeemu();
			return;

		case GDK_F12:
			create_loaddlg();
			return;
	}			
	emkeyin((unsigned short) event->keyval, 0xFF);
}

void keyout(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	extern struct hardware *hw;

	emkeyup((unsigned short) event->keyval, 0xFF);
}

void create_Screen(byte showkeyb)
{
	GtkWidget *gpix;
	FILE *rfile;

	lcdshow = 0;
	if (greylcd != NULL) free(greylcd);
	if (!(greylcd = malloc(3 * iWidth * iHeight))) {
		fprintf(stderr, "Insufficient Memory\n");
		exit(-1);
	}

	if ((rfile = fopen(ashare("tilem.bin"), "r")))
		fread(greylcd, 1, 3 * iWidth * iHeight, rfile);

	gdk_rgb_init();

	gpix = gtk_drawing_area_new();
		gtk_drawing_area_size(GTK_DRAWING_AREA(gpix), iWidth, iHeight);
	gtk_widget_show(gpix);

	Screen = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_title(GTK_WINDOW(Screen), "TilEm v0.7 Beta");
		gtk_window_set_position(GTK_WINDOW(Screen), GTK_WIN_POS_CENTER);

		gtk_widget_set_events(Screen, GDK_KEY_RELEASE_MASK);

		if (showkeyb)
			create_keyb(Screen, gpix);
		else
			gtk_container_add(GTK_CONTAINER(Screen), gpix);
		create_menus(Screen);

	timerid = gtk_timeout_add(50, (GtkFunction) gpix_refresh, gpix);
/*	gtk_signal_connect (GTK_OBJECT (gpix), "expose-event", GTK_SIGNAL_FUNC (on_gpix_expose), NULL);*/

	gtk_signal_connect(GTK_OBJECT(Screen), "destroy", GTK_SIGNAL_FUNC(exit_emulator), NULL);
	gtk_signal_connect(GTK_OBJECT(Screen), "key_press_event", GTK_SIGNAL_FUNC(keyin), NULL);
	gtk_signal_connect(GTK_OBJECT(Screen), "key_release_event", GTK_SIGNAL_FUNC(keyout), NULL);

	gtk_widget_show(Screen);
}

static gboolean gpix_refresh (GtkWidget *widget)
{
	unsigned char xln, yln, i;
	unsigned long t;
	FILE *rfile;

	if (!(G_TRYLOCK(scrref))) return(TRUE);

	if (!(lcdactive)) {
		if ((rfile = fopen(ashare("tilem.bin"), "r")))
			fread(greylcd, 1, 3 * iWidth * iHeight, rfile);
	}

	if ((lcdshow) && (lcdactive)) {
		for (yln = 0; yln < rHeight; yln++) {
			for (xln = 0; xln < rWidth; xln++) {
				if (((*(lcdbuf + (yln * rWidth + xln) / 8) << (xln % 0x08)) & 0x80) == 0x80)
					t = 0x000000;
				else
					t = 0xB5BDBD;

				for (i =  0; i < 3; i++)
					*(i + greylcd + 3 * (2 * yln * iWidth +  2 * xln)) = \
						*(i + greylcd + 3 * (2 * yln * iWidth + 2 * xln + 1)) = \
						*(i + greylcd + 3 * (2 * yln * iWidth + 2 * xln + iWidth)) = \
						*(i + greylcd + 3 * (2 * yln * iWidth + 2 * xln + 1 + iWidth)) = (t >> 8 * i) & 0xFF;
			}
		}
	}

	gdk_draw_rgb_image(widget->window, widget->style->fg_gc[GTK_STATE_NORMAL], 0, 0, iWidth, iHeight, GDK_RGB_DITHER_MAX, greylcd, 3 * iWidth);
	return(TRUE);
}
